@@ -1,5 +1,5 @@
-# WordNet::Extend::Locate.pm version 0.020
-# Updated: 11/08/16
+# WordNet::Extend::Locate.pm version 0.030
+# Updated: 01/16/17
 #                                           
 # Ted Pedersen, University of Minnesota Duluth             
 # tpederse at d.umn.edu
@@ -50,7 +50,7 @@ use WordNet::Extend::Locate;
 
  print "Finding location for 'dog noun withdef.1 man's best friend'\n"; 
 
- @location = $insert->locate("dog\tnoun\twithdef.1\tman\'s best friend");
+ @location = @{$locate->locate("dog\tnoun\twithdef.1\tman\'s best friend")};
 
  print "Location found: @location\n";
 
@@ -68,6 +68,8 @@ use WordNet::QueryData;
 use Getopt::Long;
 use File::Spec;
 use Lingua::Stem;
+use Lingua::EN::Tagger;
+use WordNet::Similarity::vector;
 #use List::Util;
 
 our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS);
@@ -80,11 +82,13 @@ our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS);
 
 @EXPORT = ();
 
-$VERSION = '0.020';
+$VERSION = '0.030';
 
 #**************Variables**********************
 $wn = WordNet::QueryData->new; #to be used to access data from wordnet
 $stemmer = Lingua::Stem->new; #used to stem words for better overlaps etc.
+$tagger = Lingua::EN::Tagger->new; #used to tag words' pos for similarity measure
+$measure = WordNet::Similarity::vector->new ($wn); #used to measure similarity for Similarity
 @wordNetNouns; #stores all words for noun sense from wordnet
 @wordNetVerbs; #stores all words for verb sense from wordnet
 %wnGlosses = ();
@@ -103,10 +107,10 @@ $useHypeGlosses = 1; #Toggle for use of hypernym glosses in comparisons.
 $useHypoGlosses = 1; #Toggle for use of hyponym glosses in comparisons.
 $useSynsGlosses = 1; #Toggle for use of synset glosses in comparisons.
 $bonus = 10; #Bonus to be used for lemmas that contain the new lemma. Can be set with setBonus();
-$refineSense = 1; #Toggle for use of refineSense() method, default on.
+$refineSense = 0; #Toggle for use of refineSense() method, default on.
 $help = 0;
 $scoringMethod = 'baseline';
-@scoringMethods = ('baseline', 'BwS');
+@scoringMethods = ('baseline', 'BwS', 'Similarity');
 $stemming = 0; #Toggle for stemming on or off.
 $stemmed = 0; #flag for use in BwS
 #*********************************************
@@ -236,18 +240,17 @@ sub locate()
     {
 	$base = 1;
     }
-    if($preProcessed == 1)
+
+    #if preProcessing() hasn't been called, call it.
+    if($preProcessed == 0)
     {
-	my @inLemma = split("\t", $_[$base]); #stores lemma as formatted above
-	my @outLemma = @{processLemma(\@inLemma)};
-	return \@outLemma;
-    }
-    else
-    {
-	my $self = shift;
-	$self->{error} = 2;
-	$self->{errorString} = "preProcesssing() must be run once before calling locate()";
+	preProcessing();
     }
+    
+    my @inLemma = split("\t", $_[$base]); #stores lemma as formatted above
+    my @outLemma = @{processLemma(\@inLemma)};
+    return \@outLemma;
+    
 }
 
 =item $obj->stopList($newStopList)
@@ -668,6 +671,10 @@ sub scoreSense()
     {
 	$score = BwS(\@inLemma, $curSense);
     }
+    if($scoringMethod eq "Similarity")
+    {
+	$score = Similarity(\@inLemma, $curSense);
+    }
 
     return $score;
 }   
@@ -707,6 +714,88 @@ sub setScoreMethod()
     
 }
 
+=item $obj->Similarity(@inLemma, $compareSense)
+
+Calculates a score for the passed sense and returns
+that score.
+
+Parameters: the in lemma in array form 
+(lemma, part-of-speech, item-id, definition, def source)
+and the sense that the lemma is being compared to.
+
+Returns: a score of how related the im lemma is to the 
+compareSense.
+
+=cut
+
+sub Similarity()
+{
+    my $base = 0;
+    if(scalar @_ == 3)#checks if method entered by object.
+    {
+	$base = 1;
+    }
+
+    my @inLemma = @{$_[$base]};
+    $base++;
+    my $curSense = $_[$base];
+
+    my $def = @inLemma[3];
+
+    #split definition and stem the words
+    my @listDef =split(' ', $def);
+    my @defStemmed = @{$stemmer->stem(@listDef)};
+
+    #join definition back together and tag with pos
+    $def = join(' ', @defStemmed);
+    my $tagged = $tagger->add_tags($def);
+
+    #split the tagged definition for individual word processing
+    @tagArray = split(' ', $tagged);
+    my @similar = ();
+
+    #step through each tagged word and find the first sense in wordnet, then add that to the @similar list
+    foreach my $cur (@tagArray)
+    {
+	my $pos = '';
+	if($cur =~ /<nn>.*/)
+	{
+	    $pos = 'n';
+	}
+	else
+	{
+	    if($cur =~ /<vbp>.*/)
+	    {
+		$pos = 'v';
+	    }
+	    else
+	    {
+		if($cur =~ /<jj>.*/)
+		{
+		    $pos = 'a';
+		}
+	    }
+	}
+
+	if(length $pos == 1)
+	{
+	    $cur =~ s/<[nvj\/].{1,3}>//g;
+	    @wnQuery = $wn->querySense("$cur#$pos");
+	    push @similar, $wnQuery[0];
+	}
+    }
+
+    my $score = 0;
+
+    foreach my $curSim (@similar)
+    {
+	my $value = $measure->getRelatedness("$curSense", "$curSim");
+	$score = $score + $value;
+    }
+    
+    return $score;
+}
+
 =item $obj->BwS(@inLemma, $compareSense)   
 
 Calculates a score for the passed sense and returns
